
ULib.ucl = {}
local ucl = ULib.ucl 




ucl.users = {}
ucl.groups = {}
ucl.authed = {}
ucl.awaitingauth = {}
ucl.callbacks = {}





function ucl.query( ply, command, hide )
	if not ply:IsValid() or (not hide and ply:IsListenServerHost()) then return true end 

	local match 
	if not command:find( " " ) then
		match = ULib.concommands[ command ]
	else
		local prefix_list = string.Explode( " ", command )
		local num = #prefix_list
		match = ULib.subconcommands

		for i=1, num do 
			match = match[ prefix_list[ i ] ]
			if not match then
				break
			end
		end
	end

	if not match then 
		return ply:IsUserGroup( ULib.DEFAULT_ACCESS )
	end
	
	if not ucl.authed[ ply ] then
		return ply:IsUserGroup( match.access )
	end

	
	local denies = { ucl.authed[ ply ].deny }
	local allows = { ucl.authed[ ply ].allow }
	
	for _, v in pairs( ucl.authed[ ply ].groups ) do
		if ucl.groups[ v ] then
			table.insert( denies, ucl.groups[ v ].deny )
			table.insert( allows, ucl.groups[ v ].allow )			
		end
	end
	
	for _, v in pairs( denies ) do
		if table.HasValue( v, command ) then return false end
	end
	
	for _, v in pairs( allows ) do
		if table.HasValue( v, command ) then return true end
	end

	
	return ply:IsUserGroup( match.access )
end






local meta = FindMetaTable( "Player" )
if not meta then return end

function meta:IsAdmin()
	if self:IsSuperAdmin() then return true end
	if self:IsUserGroup( "admin" ) then return true end

	return false
end

function meta:IsSuperAdmin()
	
	if SERVER and self:IsListenServerHost() then return true end

	return self:IsUserGroup( "superadmin" )
end

function meta:IsUserGroup( name )
	if name == ULib.ACCESS_ALL then return true end 
	if name == ULib.ACCESS_NONE then return false end 

	if not ucl.authed[ self ] then return false end

	
	if name == ULib.ACCESS_OPERATOR then
		if table.HasValue( ucl.authed[ self ].groups, ULib.ACCESS_OPERATOR ) then return true end
		if table.HasValue( ucl.authed[ self ].groups, ULib.ACCESS_ADMIN ) then return true end
		if table.HasValue( ucl.authed[ self ].groups, ULib.ACCESS_SUPERADMIN ) then return true end
		return false
	end

	if name == ULib.ACCESS_ADMIN then
		if table.HasValue( ucl.authed[ self ].groups, ULib.ACCESS_ADMIN ) then return true end
		if table.HasValue( ucl.authed[ self ].groups, ULib.ACCESS_SUPERADMIN ) then return true end
		return false
	end

	if name == ULib.ACCESS_SUPERADMIN then
		if table.HasValue( ucl.authed[ self ].groups, ULib.ACCESS_SUPERADMIN ) then return true end
		return false
	end

	
	return table.HasValue( ucl.authed[ self ].groups, name )
end

function meta:SetUserGroup( group )
	if not ULib.UCL_LOAD_DEFAULT then return end 
	
	if table.HasValue( ucl.authed[ self ].groups ) then return end 
	
	table.insert( ucl.authed[ self ].groups, group )
end

function meta:query( command, hide )
	return ULib.ucl.query( self, command, hide )
end






function ucl.callCallbacks( ply )
	for _, fn in ipairs( ucl.callbacks ) do
		local hasAccess = not not ucl.authed[ ply ] 
		pcall( fn, ply, hasAccess )
	end
end



function ucl.addAccessCallback( fn )
	table.insert( ucl.callbacks, fn )
end

if CLIENT then 

	function ucl.rcvUserData( um )
		local ply = um:ReadEntity()
		ucl.authed[ ply ] = ULib.umsgRcv( um )
	end
	usermessage.Hook( "ULibUserUCL", ucl.rcvUserData )
	
	function ucl.rcvGroupData( um )
		local group = um:ReadString()
		ucl.groups[ group ] = ULib.umsgRcv( um )
	end
	usermessage.Hook( "ULibGroupUCL", ucl.rcvGroupData )
	
	function ucl.rcvFinished( um )
		ucl.callCallbacks( LocalPlayer() )
	end
	usermessage.Hook( "ULibFinishedUCL", ucl.rcvFinished )

end